#include "sensors.hpp"
#include "drivers/i2c.h"
#include "FreeRTOS/FreeRTOS.h"
#include "FreeRTOS/task.h"
#include "osHandles.h"
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space

typedef enum
{
	leftSensorAddr = 0xE0, rightSensorAddr = 0xF2, middleSensorAddr = 0xE2
} sensorAddrType;


void getLeftAndRightAndMiddleSensorReadings(int* left, int* right, int* middle);


void getLeftAndRightAndMiddleSensorReadings(int* left, int* right, int* middle)
{
	typedef union {
		struct {
			char low;
			char high;
		};
		unsigned int _32Bit;
	}ShortSplitterType;
	ShortSplitterType value;

	const char triggerValue = 81;
	unsigned int NotDetectedAnyObstacle = 0;
	unsigned int SetAnyFarRange = 300;

	// Trigger the left and the right sensor
	i2cWriteDeviceRegister(leftSensorAddr, 0, triggerValue);
	while(!i2cIsDeviceReady(leftSensorAddr)) { vTaskDelay(5); }
	value._32Bit = 0;
	value.high = i2cReadDeviceRegister(leftSensorAddr, 2); //High Byte
	value.low  = i2cReadDeviceRegister(leftSensorAddr, 3); //Low Byte
	*left = value._32Bit;


	i2cWriteDeviceRegister(rightSensorAddr, 0, triggerValue);
	while(!i2cIsDeviceReady(rightSensorAddr)){ vTaskDelay(5); }
	value._32Bit = 0;
	value.high = i2cReadDeviceRegister(rightSensorAddr, 2); //High Byte
	value.low  = i2cReadDeviceRegister(rightSensorAddr, 3); //Low Byte
	*right = value._32Bit;


	i2cWriteDeviceRegister(middleSensorAddr, 0, triggerValue);
	while(!i2cIsDeviceReady(middleSensorAddr))  { vTaskDelay(5); }
	value._32Bit = 0;
	value.high = i2cReadDeviceRegister(middleSensorAddr, 2); //High Byte
	value.low  = i2cReadDeviceRegister(middleSensorAddr, 3); //Low Byte
	if(value._32Bit == NotDetectedAnyObstacle)
	*middle = SetAnyFarRange;
	else
		*middle = value._32Bit;
}

/******************************************************************************
 ** Function name:		SensorUpdate()
 **
 ** Descriptions:		Update Left, Middle, and Right Sensors.
 ******************************************************************************/
void SensorUpdate(void* p)
{
	OSHANDLES *osHandles = (OSHANDLES*) p;
	portTickType prevWakeTime = xTaskGetTickCount();

	for (;;)
	{
		int left = 0;
		int right = 0;
		int middle = 0;
		getLeftAndRightAndMiddleSensorReadings(&left, &right, &middle);

		// Assuming sensors got triggered, simply grab their values
		osHandles->sensorValues.left = left;
		osHandles->sensorValues.right = right;
		osHandles->sensorValues.middle = middle;
		//osHandles->sensorValues.middle = getUltrasonicMiddleI2C();
		rprintf("% 5u % 5u % 5u\n", left, middle, right);

		const int minimumDelayMsBeforeTriggeringSensors = 70;
		vTaskDelayUntil(&prevWakeTime, minimumDelayMsBeforeTriggeringSensors);
	}
}
